package com.ecpei.common.tools;

import android.app.Activity;
import android.content.Context;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Point;
import android.hardware.Camera;
import android.os.Build;
import android.provider.Settings;
import android.support.annotation.NonNull;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Display;
import android.view.KeyCharacterMap;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.WindowManager;

import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.Nullable;

public class CommonToolsModeule extends ReactContextBaseJavaModule {

    public static final String REACTCLASSNAME = "CommonToolsModule";
    public static Context mContext;

    @Override
    public String getName() {
        return REACTCLASSNAME;
    }

    public CommonToolsModeule(ReactApplicationContext reactContext) {
        super(reactContext);
        mContext = reactContext;

    }

    /**
     *  部分手机无法判断权限是否被拒
     *  由此判断摄像头是否可用
     * @param callback
     */
    @ReactMethod
    public void isPermissionCompatibleCamera(Callback callback){
        boolean isCanUse = true;
        Camera mCamera = null;
        try {
            mCamera = Camera.open();
            Camera.Parameters mParameters = mCamera.getParameters(); //针对魅族手机
            mCamera.setParameters(mParameters);
        } catch (Exception e) {
            isCanUse = false;
        }
        if (mCamera != null) {
            try {
                mCamera.stopPreview();
                mCamera.setPreviewDisplay(null);
                mCamera.setPreviewCallback(null);
                mCamera.release();
            } catch (Exception e) {
                e.printStackTrace();
                isCanUse = false;
            }
        }
        callback.invoke(isCanUse);
    }

    /**
     * 获取系统高度
     * @return
     */
    @Nullable
    @Override
    public Map<String, Object> getConstants() {
        Map result = new HashMap<String,Object>();
        int height = getFullActivityHeight(getCurrentActivity());
        result.put("ScreenHeight",px2dp(height));
        return result;
    }

    //将px转换为与之相等的dp
    public int px2dp(float dipValue){
        final float scale = mContext.getResources().getDisplayMetrics().density;
        return (int)(dipValue / scale + 0.5f);
    }

    /**
     * 根据手机判断获取实际手机高度
     * @param context
     * @return
     */
    public int getFullActivityHeight(@Nullable Context context) {
        if (!isAllScreenDevice()) {
            //判断不是全面屏手机
            Log.e("android_message","不是全屏手机");
            return getScreenHeight(context);
        }else{
            if(OSUtil.isMiui()){  //判断是小米系统
                if(Settings.Global.getInt(context.getContentResolver(), "force_fsg_nav_bar", 0) != 0){
                    //开启手势，不显示虚拟键
                    // Log.e("android_message","是小米手机关闭了虚拟按键");
                    return getScreenRealHeight(context);
                }else {
                    // Log.e("android_message","是小米手机开启着虚拟按键");
                    return getScreenRealHeight(context) - getVirtualButtonsHeigth();
                }
            } else if (OSUtil.isEmui()) { //判断华为手机
                if ( isNavigationBarShow(getCurrentActivity())){
                    // Log.e("android_message","是华为手机开启着虚拟按键");
                    return getScreenRealHeight(context) - getVirtualButtonsHeigth();
                }else{
                    // Log.e("android_message","是华为手机关闭着虚拟按键");
                    return getScreenRealHeight(context);
                }
            } else {
                if ( isNavigationBarShow(getCurrentActivity())){
                    // Log.e("android_message","其他手机开启着虚拟按键");
                    return getScreenRealHeight(context) - getVirtualButtonsHeigth();
                }else{
                    // Log.e("android_message","其他手机关闭了虚拟按键");
                    return getScreenRealHeight(context);
                }
            }
        }
    }

    /**
     * 判断是否开启或关闭底部虚拟按键
     * （华为全面屏手机可用）
     * @param activity
     * @return
     */
    public static boolean isNavigationBarShow(Activity activity){
        //虚拟键的view,为空或者不可见时是隐藏状态
        View view = activity.findViewById(android.R.id.navigationBarBackground);
        if(view == null){
            return false;
        }
        int visible = view.getVisibility();
        if(visible == View.GONE || visible == View.INVISIBLE){
            return false ;
        }else{
            return true;
        }
    }


    private static final int PORTRAIT = 0;
    private static final int LANDSCAPE = 1;
    @NonNull
    private volatile static Point[] mRealSizes = new Point[2];

    /**
     * 获取手机虚拟按键的高度
     * @return
     */
    private int getVirtualButtonsHeigth(){
        Resources res = getCurrentActivity().getResources();
        int resourceId = res.getIdentifier("navigation_bar_height", "dimen", "android");
        if (resourceId > 0) {
            return res.getDimensionPixelSize(resourceId);
        }
        return 0;
    }

    /**
     * 获取全面屏手机的全部高度
     * @param context
     * @return
     */
    public int getScreenRealHeight(Context context){
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR1) {
            return getScreenHeight(context);
        }

        int orientation = context != null
                ? context.getResources().getConfiguration().orientation
                : context.getResources().getConfiguration().orientation;
        orientation = orientation == Configuration.ORIENTATION_PORTRAIT ? PORTRAIT : LANDSCAPE;

        if (mRealSizes[orientation] == null) {
            WindowManager windowManager = context != null
                    ? (WindowManager) context.getSystemService(Context.WINDOW_SERVICE)
                    : (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
            if (windowManager == null) {
                return getScreenHeight(context);
            }
            Display display = windowManager.getDefaultDisplay();
            Point point = new Point();
            display.getRealSize(point);
            mRealSizes[orientation] = point;
        }
        return mRealSizes[orientation].y;
    }

    /**
     * 获取非全面屏手机的高度
     * @param context
     * @return
     */
    public static int getScreenHeight(@Nullable Context context) {
        if (context != null) {
            return context.getResources().getDisplayMetrics().heightPixels;
        }
        return 0;
    }

    private volatile static boolean mHasCheckAllScreen;
    private volatile static boolean mIsAllScreenDevice;

    /**
     * 判断手机是否是全面屏手机
     * @return
     */
    public boolean isAllScreenDevice() {
        if (mHasCheckAllScreen) {
            return mIsAllScreenDevice;
        }
        mHasCheckAllScreen = true;
        mIsAllScreenDevice = false;
        // 低于 API 21的，都不会是全面屏。。。
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            return false;
        }
        WindowManager windowManager = (WindowManager) getCurrentActivity().getSystemService(Context.WINDOW_SERVICE);
        if (windowManager != null) {
            Display display = windowManager.getDefaultDisplay();
            Point point = new Point();
            display.getRealSize(point);
            float width, height;
            if (point.x < point.y) {
                width = point.x;
                height = point.y;
            } else {
                width = point.y;
                height = point.x;
            }
            if (height / width >= 1.97f) {
                mIsAllScreenDevice = true;
            }
        }
        return mIsAllScreenDevice;
    }
}
