
import {
    ToastAndroid,
    PermissionsAndroid,
    Platform,
    NativeModules
} from 'react-native'
import Permissions from 'react-native-permissions'
import MessageManager from "./Events/MessageManager";

class Permission {


    permissionTips = Array();
    constructor() {
        //                         Type	     iOS	 Android
        // Location          	location	✔️	✔
        // Camera	            camera	✔️	✔
        // Microphone	         microphone	✔️	✔
        // Photos	             photo	✔️	✔
        // Contacts	             contacts	✔️	✔
        // Events	             event	✔️	✔
        // Bluetooth	         bluetooth	✔️	❌
        // Reminders	         reminder	✔️	❌
        // Push Notifications	notification	✔️	❌
        // Background Refresh	backgroundRefresh	✔️	❌
        // Speech Recognition	speechRecognition	✔️	❌
        // mediaLibrary	        mediaLibrary	✔️	❌
        // Motion Activity       	motion	✔️	❌
        // Storage	             storage	❌️	✔
        // Phone Call	        callPhone	❌️	✔
        // Read SMS	            readSms	❌️	✔
        // Receive SMS	           receiveSms	❌️	✔
        // 可增加权限类型
        this.permissionTips["camera"] = "请授予访问相机的权限才能使用该功能";
        this.permissionTips["storage"] = "请授予访问本地文件的权限才能使用该功能";
    }

    permissionResult(response, notTip, success, fail) {
        switch (response) {
            case "authorized":
                success && success();
                break;
            case "denied":
            case "restricted":
                if (notTip)
                    this.show(notTip)
            default:
                fail && fail();
                // this.props.navigation && this.props.navigation.goBack();
                break;

        }
    }

    /**
         * 获取权限
         * @param {权限类型，可为数组} types
         * @param {权限失败提示，可为数组} tips
         */
        requestPermission(types, tips) {
            var _this = this;
            return new Promise(async function (resolve, reject) {
                if (!types) reject({ status: -1, msg: "参数错误" });
                if (!tips) tips = Array();
                // alert(Array.isArray(types));
                if (typeof types === "string") {
                    types = [types];
                }
                if (typeof tips === "string") {
                    tips = [tips];
                }
                let isFail = true;
                let _num = 0;
                for (const item of types) {
                    console.log("item1", item);
                    if (Platform.OS === "ios" && (item == "storage" || item == "callPhone" || item == "readSms" || item == "receiveSms")) {
                        _num++;
                        continue;
                    };
                    if (isFail)
                        await Permissions.request(item).then(response => {
                            _this.permissionResult(response, tips[_num] ? tips[_num] : _this.permissionTips[item], function () {
                                console.log("item2", types.length == _num + 1);
                            }, function () {
                                isFail = false;
                                console.log("item3", item);
                                if (types.length == _num + 1) {
                                    return;
                                }
                            });
                        });
                    else {
                        break;
                    }
                    _num++;
                }
                if (isFail){
                    if (Platform.OS == 'android' && types.toString().indexOf("camera") > -1) {
                        NativeModules.CommonToolsModule.isPermissionCompatibleCamera((isCanUse) => {
                            if(isCanUse){
                                resolve({ status: 1 });
                            }else{
                                _this.show(_this.permissionTips['camera'])
                                reject({ status: 99, msg: "授权失败" });
                            }
                        });
                    }else{
                        resolve({ status: 1 });
                    }
                } else{
                    reject({ status: 99, msg: "授权失败" });
                }
            });
        }



    requestCarmeraPermission() {
        return new Promise(async function (resolve, reject) {
            resolve(true); return;
            //iOS不走android判断逻辑
            if (Platform.OS == 'ios') {
                resolve(true);
                return;
            }
            try {
                const granted = await PermissionsAndroid.request(
                    PermissionsAndroid.PERMISSIONS.CAMERA,
                    {
                        //第一次请求拒绝后提示用户你为什么要这个权限
                        'title': '权限获取提示',
                        'message': '请授予访问相机的权限才能使用该功能'
                    }
                )
                if (granted === PermissionsAndroid.RESULTS.GRANTED) {
                    resolve(true);
                } else {
                    reject(false);
                    show("相机权限获取失败")
                }
            } catch (err) {
                show(err.toString())
                reject(false);
            }
        });
    }


    requestReadWritePermission() {
        return new Promise(async function (resolve, reject) {
            resolve(true); return;
            //iOS不走android判断逻辑
            if (Platform.OS == 'ios') {
                resolve(true);
                return;
            }
            try {
                const granted = await PermissionsAndroid.request(
                    PermissionsAndroid.PERMISSIONS.WRITE_EXTERNAL_STORAGE
                )
                if (granted === PermissionsAndroid.RESULTS.GRANTED) {
                    resolve(true);
                } else {
                    reject(false);
                    show("权限获取失败")
                }
            } catch (err) {
                show(err.toString())
                reject(false);
            }
        });
    }


    show(data) {
        Platform.OS == "android" ? ToastAndroid.show(data, ToastAndroid.SHORT) : MessageManager.fireEvent(data)
    }
}

export default new Permission();