
/**
 * 屏幕工具类
 * ui设计基准,iphone 6
 * width:750
 * height:1334
 */
import {Dimensions, PixelRatio} from 'react-native'
import theme from "./base-theme";
export let screenW = theme.deviceWidth;
export let screenH = theme.deviceHeight;
const fontScale = PixelRatio.getFontScale();
export let pixelRatio = PixelRatio.get();
//像素密度
export const DEFAULT_DENSITY = 2;
//px转换成dp
//以iphone6为基准,如果以其他尺寸为基准的话,请修改下面的750和1334为对应尺寸即可.
const w2 = 750 / DEFAULT_DENSITY;
//px转换成dp
const h2 = 1334 / DEFAULT_DENSITY;

// iPhoneX
const X_WIDTH = 375;
const X_HEIGHT = 812;

/**
 * 设置字体的size（单位px）
 * @param size 传入设计稿上的px
 * @returns {Number} 返回实际sp
 */
export function setSpText(size: Number) {
    let scaleWidth = screenW / w2;
    let scaleHeight = screenH / h2;
    let scale = Math.min(scaleWidth, scaleHeight);
    size = Math.round((size * scale + 0.5));
    return size / DEFAULT_DENSITY;
}

/**
 * 屏幕适配,缩放size
 * @param size
 * @returns {Number}
 */
export function scaleSize(size: Number) {
    let scaleWidth = screenW / w2;
    let scaleHeight = screenH / h2;
    let scale = Math.min(scaleWidth, scaleHeight);
    size = Math.round((size * scale + 0.5));
    return size / DEFAULT_DENSITY;
}

/**
 * 手机号正则校验
 * @param phoneNumber
 * @returns {BOOL}
 */
export function validatemobile(phoneNumber:string) {
    if (phoneNumber.length == 0) {
        return false;
    }
    if (phoneNumber.length != 11) {
        return false;
    }
    var PATTERN_CHINAMOBILE = /^1(3[4-9]|5[012789]|8[23478]|4[7]|7[8])\d{8}$/; //移动号
    var PATTERN_CHINAUNICOM = /^1(3[0-2]|5[56]|8[56]|4[5]|7[6])\d{8}$/; //联通号
    var PATTERN_CHINATELECOM = /^1(3[3])|(8[019])\d{8}$/; //电信号
    if (PATTERN_CHINAUNICOM.test(phoneNumber)) {
        return true;
    } else if (PATTERN_CHINAMOBILE.test(phoneNumber)) {
        return true;
    } else if (PATTERN_CHINATELECOM.test(phoneNumber)) {
        return true;
    }else {
        return false;
    }
  }
  
  export function validate(phoneNumber:string){
    //只能输入数字
    var reg = new RegExp("^[0-9]*$");
    if(!reg.test(phoneNumber)){
        return false;
    }
    //匹配数字字符
    if(!/^[0-9]+$/.test(phoneNumber)){
        return false;
    }
    return true;
  }

//
export function testEnglishOrNumber(text:string){
    //只能输入由数字和26个英文字母组成的字符串
    const reg = new RegExp('^[A-Za-z0-9]+$')
    if(reg.test(text)){
        return true;
    }
    return false
}

//
export function testChinese(text:string){
    //检测匹配中文
    const reg = new RegExp('^[\u4e00-\u9fa5]*$');
    if(reg.test(text)){
        return true
    }
    return false
}

export function checkPassword(text:string){
    // var  password =  /(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z]{6,20}/ ;
    // /^(?=.*\d)(?=.*[a-zA-Z])(?=.*[~!@#$%^&*])[\da-zA-Z~!@#$%^&*]{8,}$/
    // console.log("获取输入字符串",text);
    // if (password.test(text)) {
    //     return true;
    // }else {
    //     return false;
    // }

    var str = text;
    if (str == null || str.length <6  || str.length >20) {
        return false;
    }
    var reg1 = new RegExp(/^[0-9A-Za-z]+$/);
    if (!reg1.test(str)) {
        return false;
    }
    var reg = new RegExp(/[A-Za-z].*[0-9]|[0-9].*[A-Za-z]/);
    if (reg.test(str)) {
        return true;
    } else {
        return false;
    }

}

function CheckPassWord(password) {//必须为字母加数字且长度不小于8位
    var str = password;
     if (str == null || str.length <8) {
         return false;
     }
     var reg1 = new RegExp(/^[0-9A-Za-z]+$/);
     if (!reg1.test(str)) {
         return false;
     }
     var reg = new RegExp(/[A-Za-z].*[0-9]|[0-9].*[A-Za-z]/);
     if (reg.test(str)) {
         return true;
     } else {
         return false;
     }
 }
export default {
    //正则表
    regList: {
        SmsCode: /^\d{6}$/,
        /*短信正则*/
        Password: /(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z]{6,20}/,
        /*密码正则*/
        Mobile: /^[1][34578][0-9]{9}$/,
        /*手机号正则*/
        RealName: /^[\u4e00-\u9fa5 ]{2,10}$/,
        /*真实姓名正则*/
        WeiXin: /^[a-zA-Z\d_]{5,19}$/,
        BankNum: /^\d{10,19}$/,
        /*银行卡号正则*/
        Money: /^([1-9]\d*|0)$/,
        /*钱正则*/
        Mail: /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/ /*邮件正则*/
    },

} 
