import MessageManager,{ MessageType, Message } from "./Events/MessageManager";
import NetWorkStatusManager, { ConnectStatus } from "./NetWorkStatusManager";
import * as URL from "url";
import { EventEmitterCenter } from "./EventEmitter";
import { DeviceEventEmitter } from "react-native";
const codeMessage = {
    200: '服务器成功返回请求的数据',
    201: '新建或修改数据成功。',
    202: '一个请求已经进入后台排队（异步任务）',
    203: 'token已失效',
    204: '删除数据成功。',
    400: '发出的请求有错误，服务器没有进行新建或修改数据,的操作。',
    401: '用户没有权限（令牌、用户名、密码错误）。',
    403: '用户得到授权，但是访问是被禁止的。',
    404: '发出的请求针对的是不存在的记录，服务器没有进行操作',
    406: '请求的格式不可得。',
    410: '请求的资源被永久删除，且不会再得到的。',
    422: '当创建一个对象时，发生一个验证错误。',
    500: '服务器发生错误，请检查服务器',
    502: '网关错误',
    503: '服务不可用，服务器暂时过载或维护',
    504: '网关超时',
    518: "解析报错",
    600: "超时"
};
export class RequestOption {
    url = "";
    option = null;
    timeout = 10000
    constructor(url, ops, timeout) {
        this.url = url;
        this.option = ops;
        this.timeout = timeout;
    }
}
export class Interceptor {
    constructor() {
    }
    interceptor(request) {
        return request;
    }
    clear() { }
}
class _NetWorkManager {
    interceptors = [];
    static TimeOut = 10000
    isConnet = true
    constructor() {
        NetWorkStatusManager.isConnectSubject.subscribe((isConnet) => {
            this.isConnet = isConnet == ConnectStatus.isConnect
        })
    }
    filterStatus(response) {
        if (response.status >= 200 && response.status < 300) {
            return response;
        }
        const errortext = codeMessage[response.status] || response.statusText;
        const error = new Error(errortext);
        error.name = response.status;
        error.response = response;
        throw error;
    }
    filterJSON(res) { return res.json() }

    addInterceptor(interceptor) {
        if (__DEV__ && !(interceptor instanceof Interceptor)) {
            throw new Error("Interceptor 验证错误")
        }
        this.interceptors.push(interceptor)
    }
    clearInters() {
        this.interceptors.map((inter) => { inter.clear() })
        this.interceptors = [];
    }
    _HandleInterceptor(options) {
        let _options = options;
        for (var i = 0; i < this.interceptors.length; i++) {
            _options = this.interceptors[i].interceptor(_options)
        }
        return _options;
    }
    get(url, timeout = _NetWorkManager.TimeOut) {
        url = encodeURI(url)
        timeout = this.isConnet ? timeout : 0
        options = this._HandleInterceptor(new RequestOption(url, {
            method: 'GET',
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json',

            }
        }, timeout))
        url = options.url;
        option = options.option
        timeout = options.timeout

        return Promise.race(
            [
                fetch(url, option).then(this.filterStatus).then(this.filterJSON)
                , new Promise((resolve, reject) => {
                    setTimeout(() => { reject({ status_code: 600, msg: "超时", status: 600 }) }, timeout)
                })
            ]
        ).catch((error) => {
            if (error.status_code && error.status_code == 600) {
                this._handleTimeOut()
            }
            return error
        })
    }
    post(url, { body = {}, headers = {} }, timeout = _NetWorkManager.TimeOut) {
        url = encodeURI(url)
        timeout = this.isConnet ? timeout : 0
        options = this._HandleInterceptor(new RequestOption(url, {
            method: 'POST',
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json',
                ...headers,
            },
            body
        }, timeout))
        url = options.url;
        option = { ...options.option, body: JSON.stringify(options.option.body) };
        timeout = options.timeout;
        return Promise.race([
            fetch(url, option).then(this.filterStatus).then(this.filterJSON),
            new Promise((resolve, reject) => {
                setTimeout(() => { reject({ status_code: 600, msg: "超时", status: 600 }) }, timeout)
            })
        ]).catch((error) => {
            if (error.status_code && error.status_code == 600) {
                this._handleTimeOut()
            }
            return error
        })
    }
    request(url, options = { method: "GET" }, timeout = _NetWorkManager.TimeOut) {
        _token = (typeof url == "string" && (options.method || "GET").toLocaleUpperCase() == "POST") ? options.body.token : URL.parse(encodeURI(url), true).query.token
        if (_token != undefined && _token.length <= 20) {
            return new Promise((resolve, reject) => {
                resolve({ status_code: 406, msg: "_token 传递的不对", status: -1, opstions: { url, ...options, timeout } })
            })
        }
        const defaultOptions = { credentials: 'include' };
        newOptions = { ...defaultOptions, ...options };
        timeout = this.isConnet ? timeout : 0

        options = this._HandleInterceptor(new RequestOption(url, newOptions, timeout))
        url = options.url
        newOptions = options.option
        timeout = options.timeout
        if (newOptions.method === 'POST' || newOptions.method === 'PUT') {
            newOptions.headers = {
                'Content-Type': 'multipart/form-data',
                ...newOptions.headers,
            };
            newOptions.mode = "cros";
            var formData = new FormData();
            for (var name in newOptions.body) {
                if (typeof newOptions.body[name] == "object") {
                    for (var o in newOptions.body[name]) {
                        formData.append(name + "[" + encodeURI(o) + "]", newOptions.body[name][o]);
                    }
                } else {
                    formData.append(name, newOptions.body[name]);
                }
            }
            newOptions.body = formData;
        }
        return Promise.race([
            fetch(url, { method: newOptions.method, body: newOptions.body, ...newOptions })
                .then(this.filterStatus)
                .then((response) => {
                    const data = response.json();
                    return data;
                }),
            new Promise((resolve, reject) => {
                setTimeout(() => { reject({ status_code: 600, msg: "超时", status: 600 }) }, timeout)
            })
        ])
            .then((data) => {
                if (data.status_code == 200) {
                    MessageManager.fireEvent(new Message("请求成功", MessageType.Success, null, null, url));
                } else {
                    MessageManager.fireEvent(new Message("请求失败", MessageType.Fail, null, null, url));
                }

                if (data.status == 203) {
                    //暂时发两个 兼容后去掉
                    EventEmitterCenter.emit('NativeNotice', { data: JSON.stringify({ event: "loginOut" }), event: "loginOut", reason: "fetch 203状态",url });
                    DeviceEventEmitter.emit('NativeNotice', { data: JSON.stringify({ event: "loginOut" }), event: "loginOut", reason: "fetch 203状态",url });
                }
                return data
            }).catch((error) => {
                if (error.status_code && error.status_code == 600) {
                    this._handleTimeOut()
                    return {
                        status: 1,
                        code: 600,
                        reason: "超时"
                    }
                }
                return {
                    status: 1,
                    code: 518,
                    reason: error
                }
            })
    }
    last = null
    _handleTimeOut() {
        first = this.last == null
        current = new Date()
        last = this.last || current
        if (first || current.getTime() - last.getTime() >= 60000) {
            this.last = current
            MessageManager.fireEvent(new Message("请检查网络", MessageType.FailShould, 750, null, new Error("超时")))
        }
    }
}

const NetWork = new _NetWorkManager()
export default NetWork
export const get = NetWork.get.bind(NetWork)
export const post = NetWork.post.bind(NetWork)
export const request = NetWork.request.bind(NetWork)

